import pytest 
import os
import random
import numpy as np
from random import seed
from random import randint
from click.testing import CliRunner
from aapg.main import cli
import sys

seed(1)

list_of_files = ['check_basic', 'check_branch', 'check_csr', 'check_switch_mode', 'check_exceptions', 'check_i_cache', 'check_recursion', 'check_all']


seeds = []

for _ in range(30):
    value = randint(0, 1000)
    seeds.append(value)

@pytest.fixture
def runner():
    return CliRunner()

@pytest.mark.serial
def test_print_seed(seed):
    print ("Displaying seed: "+seed)

@pytest.mark.serial
def test_setup(runner,seed):
    '''Testing setup option in serial mode'''
    result = runner.invoke(cli, ['setup','--setup_dir=work'])
    assert result.exit_code == 0

@pytest.mark.serial
def test_same_seed_gen1(runner,seed):
    '''Generating test with a seed'''
    result = runner.invoke(cli, ['gen','--setup_dir=work','--output_dir=work1','--seed={gen_seed}'.format(gen_seed=seeds[1]),'--asm_name=test1'])
    assert result.exit_code == 0

@pytest.mark.serial
def test_same_seed_gen2(runner,seed):
    '''Generating test with a seed'''
    result = runner.invoke(cli, ['gen','--setup_dir=work','--output_dir=work1','--seed={gen_seed}'.format(gen_seed=seeds[1]),'--asm_name=test2'])
    assert result.exit_code == 0

@pytest.mark.serial
def test_different_seed_gen3(runner,seed):
    '''Generating test with a seed'''
    result = runner.invoke(cli, ['gen','--setup_dir=work','--output_dir=work1','--seed={gen_seed}'.format(gen_seed=seeds[2]),'--asm_name=test3'])
    assert result.exit_code == 0

# @pytest.mark.serial
# def test_check_same_seed():
#     '''Generating another test with same seed'''
#     lines1_after_5 = None
#     lines2_after_5 = None
#     with open('work1/test1.S') as f:
#         lines1_after_5 = f.readlines()[5:]
#     with open('work1/test2.S') as f:
#         lines2_after_5 = f.readlines()[5:]
#     assert lines1_after_5 == lines2_after_5

@pytest.mark.serial
def test_check_different_seed():
    '''Check percentange of overlap between programs with different seed'''
    lines1_after_5 = None
    lines2_after_5 = None
    with open('work1/test1.S') as f:
        lines1_after_5 = f.readlines()[5:]
    with open('work1/test3.S') as f:
        lines2_after_5 = f.readlines()[5:]

    len_file_1 = len(lines1_after_5)
    len_file_2 = len(lines2_after_5)
    len_check = 0

    if len_file_1<len_file_2:
        len_check = len_file_1
    else:
        len_check = len_file_2

    lines_same = []
    for i in range(len_check):
        if lines1_after_5[i] == lines2_after_5[i]:
            lines_same.append(True)
        else:
            lines_same.append(False)

    lines_same = np.array(lines_same)
    percent_same = lines_same.mean()
    assert percent_same < 0.5

@pytest.mark.serial
def test_setup_for_clean(runner,seed):
    '''Setup directory to check clean option'''
    result = runner.invoke(cli, ['setup','--setup_dir=work_setup'])
    assert result.exit_code == 0

@pytest.mark.serial
def test_gen_for_clean(runner,seed):
    '''gen to check clean option'''
    result = runner.invoke(cli, ['gen','--setup_dir=work_setup','--output_dir=work_gen'])
    assert result.exit_code == 0

@pytest.mark.serial
def test_clean(runner,seed):
    '''gen to check clean option'''
    result = runner.invoke(cli, ['clean','--setup_dir=work_setup','--output_dir=work_gen'])
    assert result.exit_code == 0


@pytest.mark.timeout(200)
def test_basic(runner,seed):
    '''Testing check_basic.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/{config_file}'.format(config_file=list_of_files[0])])
    os.system('cp tests/ci_cd_templates/{config_file}.yaml tests/work/{config_file}/config.yaml'.format(config_file=list_of_files[0]))
    result = runner.invoke(cli, ['gen','--num_programs=10','--static_make','--setup_dir=tests/work/{config_file}'.format(config_file=list_of_files[0]),'--output_dir=tests/work/{config_file}/asm'.format(config_file=list_of_files[0])])
    try:
        out = os.system('cd tests/work/{config_file}; make'.format(config_file=list_of_files[0]))
        os.system('rm -rf tests/work/{config_file}'.format(config_file=list_of_files[0]))
    except:
        make = 0
    assert result.exit_code == 0 and make == 0


@pytest.mark.timeout(200)
def test_selfcheck(runner,seed):
    '''Testing self_checking.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/{config_file}'.format(config_file="self_checking")])
    os.system('cp tests/ci_cd_templates/{config_file}.yaml tests/work/{config_file}/config.yaml'.format(config_file="self_checking"))
    result = runner.invoke(cli, ['gen','--self_checking','--setup_dir=tests/work/{config_file}'.format(config_file="self_checking"),'--output_dir=tests/work/{config_file}/asm'.format(config_file="self_checking")])
    try:
        out = os.system('cd tests/work/{config_file}; make'.format(config_file="self_checking"))
        os.system('rm -rf tests/work/{config_file}'.format(config_file="self_checking"))
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_branch(runner,seed):
    '''Testing check_branch.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/{config_file}'.format(config_file=list_of_files[1])])
    os.system('cp tests/ci_cd_templates/{config_file}.yaml tests/work/{config_file}/config.yaml'.format(config_file=list_of_files[1]))
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/{config_file}'.format(config_file=list_of_files[1]),'--output_dir=tests/work/{config_file}/asm'.format(config_file=list_of_files[1])])
    try:
        os.system('cd tests/work/{config_file}; make'.format(config_file=list_of_files[1]))
        os.system('rm -rf tests/work/{config_file}'.format(config_file=list_of_files[1]))
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_csr(runner,seed):
    '''Testing check_basic.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/{config_file}'.format(config_file=list_of_files[2])])
    os.system('cp tests/ci_cd_templates/{config_file}.yaml tests/work/{config_file}/config.yaml'.format(config_file=list_of_files[2]))
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/{config_file}'.format(config_file=list_of_files[2]),'--output_dir=tests/work/{config_file}/asm'.format(config_file=list_of_files[2])])
    try:
        os.system('cd tests/work/{config_file}; make'.format(config_file=list_of_files[2]))
        os.system('rm -rf tests/work/{config_file}'.format(config_file=list_of_files[2]))
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_switch_mode(runner,seed):
    '''Testing check_basic.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/{config_file}'.format(config_file=list_of_files[3])])
    os.system('cp tests/ci_cd_templates/{config_file}.yaml tests/work/{config_file}/config.yaml'.format(config_file=list_of_files[3]))
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/{config_file}'.format(config_file=list_of_files[3]),'--output_dir=tests/work/{config_file}/asm'.format(config_file=list_of_files[3])])
    try:
        os.system('cd tests/work/{config_file}; make'.format(config_file=list_of_files[3]))
        os.system('rm -rf tests/work/{config_file}'.format(config_file=list_of_files[3]))
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_exceptions(runner,seed):
    '''Testing check_basic.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/{config_file}'.format(config_file=list_of_files[4])])
    os.system('cp tests/ci_cd_templates/{config_file}.yaml tests/work/{config_file}/config.yaml'.format(config_file=list_of_files[4]))
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/{config_file}'.format(config_file=list_of_files[4]),'--output_dir=tests/work/{config_file}/asm'.format(config_file=list_of_files[4])])
    try:
        os.system('cd tests/work/{config_file}; make'.format(config_file=list_of_files[4]))
        os.system('rm -rf tests/work/{config_file}'.format(config_file=list_of_files[4]))
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_i_cache(runner,seed):
    '''Testing check_basic.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/{config_file}'.format(config_file=list_of_files[5])])
    os.system('cp tests/ci_cd_templates/{config_file}.yaml tests/work/{config_file}/config.yaml'.format(config_file=list_of_files[5]))
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/{config_file}'.format(config_file=list_of_files[5]),'--output_dir=tests/work/{config_file}/asm'.format(config_file=list_of_files[5])])
    try:
        os.system('cd tests/work/{config_file}; make'.format(config_file=list_of_files[5]))
        os.system('rm -rf tests/work/{config_file}'.format(config_file=list_of_files[5]))
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_recursion(runner,seed):
    '''Testing check_basic.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/{config_file}'.format(config_file=list_of_files[6])])
    os.system('cp tests/ci_cd_templates/{config_file}.yaml tests/work/{config_file}/config.yaml'.format(config_file=list_of_files[6]))
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/{config_file}'.format(config_file=list_of_files[6]),'--output_dir=tests/work/{config_file}/asm'.format(config_file=list_of_files[6])])
    try:
        os.system('cd tests/work/{config_file}; make'.format(config_file=list_of_files[6]))
        os.system('rm -rf tests/work/{config_file}'.format(config_file=list_of_files[6]))
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_all(runner,seed):
    '''Testing check_basic.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/{config_file}'.format(config_file=list_of_files[7])])
    os.system('cp tests/ci_cd_templates/{config_file}.yaml tests/work/{config_file}/config.yaml'.format(config_file=list_of_files[7]))
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/{config_file}'.format(config_file=list_of_files[7]),'--output_dir=tests/work/{config_file}/asm'.format(config_file=list_of_files[7])])
    try:
        os.system('cd tests/work/{config_file}; make'.format(config_file=list_of_files[7]))
        os.system('rm -rf tests/work/{config_file}'.format(config_file=list_of_files[7]))
    except:
        make = 0
    assert result.exit_code == 0 and make == 0


@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_hazards_s(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_hazards_s.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_hazards_s'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_hazards_s.yaml tests/work/aapg_iclass_rv64imafdc_hazards_s/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_hazards_s','--output_dir=tests/work/aapg_iclass_rv64imafdc_hazards_s/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_hazards_s; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_hazards_s')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_recurse_med(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_recurse_med.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_recurse_med'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_recurse_med.yaml tests/work/aapg_iclass_rv64imafdc_recurse_med/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_recurse_med','--output_dir=tests/work/aapg_iclass_rv64imafdc_recurse_med/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_recurse_med; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_recurse_med')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_hazards_u(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_hazards_u.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_hazards_u'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_hazards_u.yaml tests/work/aapg_cclass_rv64imafdc_hazards_u/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_hazards_u','--output_dir=tests/work/aapg_cclass_rv64imafdc_hazards_u/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_hazards_u; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_hazards_u')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_test_all2(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_test_all2.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_test_all2'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_test_all2.yaml tests/work/aapg_cclass_rv64imafdc_test_all2/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_test_all2','--output_dir=tests/work/aapg_cclass_rv64imafdc_test_all2/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_test_all2; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_test_all2')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_branches1(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_branches1.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_branches1'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_branches1.yaml tests/work/aapg_cclass_rv64imafdc_branches1/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_branches1','--output_dir=tests/work/aapg_cclass_rv64imafdc_branches1/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_branches1; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_branches1')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_bringup_s(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_bringup_s.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_bringup_s'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_bringup_s.yaml tests/work/aapg_cclass_rv64imafdc_bringup_s/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_bringup_s','--output_dir=tests/work/aapg_cclass_rv64imafdc_bringup_s/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_bringup_s; make')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_s_recursion(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_s_recursion.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_s_recursion'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_s_recursion.yaml tests/work/aapg_cclass_rv64imafdc_s_recursion/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_s_recursion','--output_dir=tests/work/aapg_cclass_rv64imafdc_s_recursion/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_s_recursion; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_s_recursion')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_branches1_u(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_branches1_u.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_branches1_u'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_branches1_u.yaml tests/work/aapg_iclass_rv64imafdc_branches1_u/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_branches1_u','--output_dir=tests/work/aapg_iclass_rv64imafdc_branches1_u/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_branches1_u; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_branches1_u')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_test_all2(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_test_all2.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_test_all2'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_test_all2.yaml tests/work/aapg_iclass_rv64imafdc_test_all2/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_test_all2','--output_dir=tests/work/aapg_iclass_rv64imafdc_test_all2/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_test_all2; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_test_all2')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_user_s(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_user_s.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_user_s'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_user_s.yaml tests/work/aapg_iclass_rv64imafdc_user_s/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_user_s','--output_dir=tests/work/aapg_iclass_rv64imafdc_user_s/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_user_s; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_user_s')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_user(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_user.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_user'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_user.yaml tests/work/aapg_cclass_rv64imafdc_user/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_user','--output_dir=tests/work/aapg_cclass_rv64imafdc_user/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_user; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_user')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_bringup(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_bringup.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_bringup'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_bringup.yaml tests/work/aapg_iclass_rv64imafdc_bringup/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_bringup','--output_dir=tests/work/aapg_iclass_rv64imafdc_bringup/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_bringup; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_bringup')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_hazards(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_hazards.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_hazards'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_hazards.yaml tests/work/aapg_cclass_rv64imafdc_hazards/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_hazards','--output_dir=tests/work/aapg_cclass_rv64imafdc_hazards/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_hazards; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_hazards')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_user(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_user.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_user'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_user.yaml tests/work/aapg_iclass_rv64imafdc_user/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_user','--output_dir=tests/work/aapg_iclass_rv64imafdc_user/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_user; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_user')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_branches1(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_branches1.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_branches1'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_branches1.yaml tests/work/aapg_iclass_rv64imafdc_branches1/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_branches1','--output_dir=tests/work/aapg_iclass_rv64imafdc_branches1/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_branches1; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_branches1')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_exceptions_u(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_exceptions_u.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_exceptions_u'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_exceptions_u.yaml tests/work/aapg_cclass_rv64imafdc_exceptions_u/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_exceptions_u','--output_dir=tests/work/aapg_cclass_rv64imafdc_exceptions_u/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_exceptions_u; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_exceptions_u')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_hazards(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_hazards.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_hazards'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_hazards.yaml tests/work/aapg_iclass_rv64imafdc_hazards/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_hazards','--output_dir=tests/work/aapg_iclass_rv64imafdc_hazards/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_hazards; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_hazards')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_recursion_branch(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_recursion_branch.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_recursion_branch'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_recursion_branch.yaml tests/work/aapg_iclass_rv64imafdc_recursion_branch/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_recursion_branch','--output_dir=tests/work/aapg_iclass_rv64imafdc_recursion_branch/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_recursion_branch; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_recursion_branch')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_hazards_u(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_hazards_u.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_hazards_u'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_hazards_u.yaml tests/work/aapg_iclass_rv64imafdc_hazards_u/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_hazards_u','--output_dir=tests/work/aapg_iclass_rv64imafdc_hazards_u/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_hazards_u; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_hazards_u')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_exceptions_u(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_exceptions_u.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_exceptions_u'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_exceptions_u.yaml tests/work/aapg_iclass_rv64imafdc_exceptions_u/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_exceptions_u','--output_dir=tests/work/aapg_iclass_rv64imafdc_exceptions_u/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_exceptions_u; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_exceptions_u')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_branches1_s(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_branches1_s.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_branches1_s'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_branches1_s.yaml tests/work/aapg_cclass_rv64imafdc_branches1_s/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_branches1_s','--output_dir=tests/work/aapg_cclass_rv64imafdc_branches1_s/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_branches1_s; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_branches1_s')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_bringup_u(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_bringup_u.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_bringup_u'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_bringup_u.yaml tests/work/aapg_cclass_rv64imafdc_bringup_u/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_bringup_u','--output_dir=tests/work/aapg_cclass_rv64imafdc_bringup_u/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_bringup_u; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_bringup_u')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_illegal(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_illegal.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_illegal'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_illegal.yaml tests/work/aapg_iclass_rv64imafdc_illegal/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_illegal','--output_dir=tests/work/aapg_iclass_rv64imafdc_illegal/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_illegal; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_illegal')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_bringup_s(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_bringup_s.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_bringup_s'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_bringup_s.yaml tests/work/aapg_iclass_rv64imafdc_bringup_s/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_bringup_s','--output_dir=tests/work/aapg_iclass_rv64imafdc_bringup_s/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_bringup_s; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_bringup_s')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_user_s(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_user_s.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_user_s'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_user_s.yaml tests/work/aapg_cclass_rv64imafdc_user_s/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_user_s','--output_dir=tests/work/aapg_cclass_rv64imafdc_user_s/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_user_s; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_user_s')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_illegal_u(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_illegal_u.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_illegal_u'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_illegal_u.yaml tests/work/aapg_cclass_rv64imafdc_illegal_u/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_illegal_u','--output_dir=tests/work/aapg_cclass_rv64imafdc_illegal_u/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_illegal_u; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_illegal_u')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_recursion_branch(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_recursion_branch.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_recursion_branch'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_recursion_branch.yaml tests/work/aapg_cclass_rv64imafdc_recursion_branch/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_recursion_branch','--output_dir=tests/work/aapg_cclass_rv64imafdc_recursion_branch/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_recursion_branch; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_recursion_branch')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_bringup(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_bringup.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_bringup'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_bringup.yaml tests/work/aapg_cclass_rv64imafdc_bringup/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_bringup','--output_dir=tests/work/aapg_cclass_rv64imafdc_bringup/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_bringup; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_bringup')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_exceptions(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_exceptions.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_exceptions'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_exceptions.yaml tests/work/aapg_iclass_rv64imafdc_exceptions/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_exceptions','--output_dir=tests/work/aapg_iclass_rv64imafdc_exceptions/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_exceptions; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_exceptions')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_branches1_s(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_branches1_s.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_branches1_s'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_branches1_s.yaml tests/work/aapg_iclass_rv64imafdc_branches1_s/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_branches1_s','--output_dir=tests/work/aapg_iclass_rv64imafdc_branches1_s/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_branches1_s; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_branches1_s')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_s_recursion(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_s_recursion.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_s_recursion'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_s_recursion.yaml tests/work/aapg_iclass_rv64imafdc_s_recursion/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_s_recursion','--output_dir=tests/work/aapg_iclass_rv64imafdc_s_recursion/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_s_recursion; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_s_recursion')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_illegal_u(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_illegal_u.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_illegal_u'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_illegal_u.yaml tests/work/aapg_iclass_rv64imafdc_illegal_u/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_illegal_u','--output_dir=tests/work/aapg_iclass_rv64imafdc_illegal_u/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_illegal_u; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_illegal_u')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_hazards_s(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_hazards_s.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_hazards_s'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_hazards_s.yaml tests/work/aapg_cclass_rv64imafdc_hazards_s/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_hazards_s','--output_dir=tests/work/aapg_cclass_rv64imafdc_hazards_s/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_hazards_s; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_hazards_s')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_user_u(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_user_u.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_user_u'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_user_u.yaml tests/work/aapg_cclass_rv64imafdc_user_u/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_user_u','--output_dir=tests/work/aapg_cclass_rv64imafdc_user_u/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_user_u; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_user_u')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_exceptions_s(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_exceptions_s.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_exceptions_s'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_exceptions_s.yaml tests/work/aapg_cclass_rv64imafdc_exceptions_s/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_exceptions_s','--output_dir=tests/work/aapg_cclass_rv64imafdc_exceptions_s/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_exceptions_s; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_exceptions_s')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_test_all(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_test_all.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_test_all'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_test_all.yaml tests/work/aapg_cclass_rv64imafdc_test_all/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_test_all','--output_dir=tests/work/aapg_cclass_rv64imafdc_test_all/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_test_all; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_test_all')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_test_all(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_test_all.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_test_all'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_test_all.yaml tests/work/aapg_iclass_rv64imafdc_test_all/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_test_all','--output_dir=tests/work/aapg_iclass_rv64imafdc_test_all/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_test_all; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_test_all')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_recurse_low(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_recurse_low.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_recurse_low'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_recurse_low.yaml tests/work/aapg_iclass_rv64imafdc_recurse_low/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_recurse_low','--output_dir=tests/work/aapg_iclass_rv64imafdc_recurse_low/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_recurse_low; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_recurse_low')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_branches1_u(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_branches1_u.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_branches1_u'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_branches1_u.yaml tests/work/aapg_cclass_rv64imafdc_branches1_u/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_branches1_u','--output_dir=tests/work/aapg_cclass_rv64imafdc_branches1_u/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_branches1_u; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_branches1_u')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_exceptions(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_exceptions.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_exceptions'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_exceptions.yaml tests/work/aapg_cclass_rv64imafdc_exceptions/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_exceptions','--output_dir=tests/work/aapg_cclass_rv64imafdc_exceptions/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_exceptions; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_exceptions')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_illegal_s(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_illegal_s.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_illegal_s'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_illegal_s.yaml tests/work/aapg_cclass_rv64imafdc_illegal_s/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_illegal_s','--output_dir=tests/work/aapg_cclass_rv64imafdc_illegal_s/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_illegal_s; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_illegal_s')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_recurse_med(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_recurse_med.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_recurse_med'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_recurse_med.yaml tests/work/aapg_cclass_rv64imafdc_recurse_med/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_recurse_med','--output_dir=tests/work/aapg_cclass_rv64imafdc_recurse_med/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_recurse_med; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_recurse_med')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_user_u(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_user_u.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_user_u'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_user_u.yaml tests/work/aapg_iclass_rv64imafdc_user_u/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_user_u','--output_dir=tests/work/aapg_iclass_rv64imafdc_user_u/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_user_u; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_user_u')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_bringup_u(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_bringup_u.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_bringup_u'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_bringup_u.yaml tests/work/aapg_iclass_rv64imafdc_bringup_u/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_bringup_u','--output_dir=tests/work/aapg_iclass_rv64imafdc_bringup_u/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_bringup_u; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_bringup_u')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_illegal(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_illegal.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_illegal'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_illegal.yaml tests/work/aapg_cclass_rv64imafdc_illegal/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_illegal','--output_dir=tests/work/aapg_cclass_rv64imafdc_illegal/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_illegal; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_illegal')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_cclass_rv64imafdc_recurse_low(runner,seed):
    '''Testing aapg_cclass_rv64imafdc_recurse_low.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_cclass_rv64imafdc_recurse_low'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_cclass_rv64imafdc_recurse_low.yaml tests/work/aapg_cclass_rv64imafdc_recurse_low/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_cclass_rv64imafdc_recurse_low','--output_dir=tests/work/aapg_cclass_rv64imafdc_recurse_low/asm'])
    try:
        os.system('cd tests/work/aapg_cclass_rv64imafdc_recurse_low; make')
        os.system('rm -rf tests/work/aapg_cclass_rv64imafdc_recurse_low')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_exceptions_s(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_exceptions_s.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_exceptions_s'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_exceptions_s.yaml tests/work/aapg_iclass_rv64imafdc_exceptions_s/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_exceptions_s','--output_dir=tests/work/aapg_iclass_rv64imafdc_exceptions_s/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_exceptions_s; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_exceptions_s')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_illegal_s(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_illegal_s.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_illegal_s'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_illegal_s.yaml tests/work/aapg_iclass_rv64imafdc_illegal_s/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_illegal_s','--output_dir=tests/work/aapg_iclass_rv64imafdc_illegal_s/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_illegal_s; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_illegal_s')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0


@pytest.mark.timeout(200)
def test_aapg_iclass_rv64imafdc_fpu_hazards(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_fpu_hazards.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_fpu_hazards'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_fpu_hazards.yaml tests/work/aapg_iclass_rv64imafdc_fpu_hazards/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_fpu_hazards','--output_dir=tests/work/aapg_iclass_rv64imafdc_fpu_hazards/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_fpu_hazards; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_fpu_hazards')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0

@pytest.mark.timeout(200)

def test_aapg_iclass_rv64imafdc_while_test(runner,seed):
    '''Testing aapg_iclass_rv64imafdc_while_test.yaml config file in parallel mode'''
    make = 0
    input_seed=int(seed)
    runner.invoke(cli, ['setup','--setup_dir=tests/work/aapg_iclass_rv64imafdc_while_test'])
    os.system('cp tests/ci_cd_templates/core_configs/aapg_iclass_rv64imafdc_while_test.yaml tests/work/aapg_iclass_rv64imafdc_while_test/config.yaml')
    result = runner.invoke(cli, ['gen','--num_programs=2','--static_make','--seed={}'.format(input_seed),'--setup_dir=tests/work/aapg_iclass_rv64imafdc_while_test','--output_dir=tests/work/aapg_iclass_rv64imafdc_while_test/asm'])
    try:
        os.system('cd tests/work/aapg_iclass_rv64imafdc_while_test; make')
        os.system('rm -rf tests/work/aapg_iclass_rv64imafdc_while_test')
    except:
        make = 0
    assert result.exit_code == 0 and make == 0



perl_file = """
#!/usr/bin/env perl
use v5.10.0;
use warnings;
srand($ARGV[1]);
my @illegal_instr_list = ();
foreach my $count (0 .. 100) {
    my $d = int(rand(4294967295));
    my $hex = sprintf("0x%X", $d);
    my $illegal = `echo "DASM(0x$hex)" | spike-dasm`;
    if ($illegal =~ /un/ && $illegal !~ /counter/) {
      push @illegal_instr_list, ".word $hex";
      push @illegal_instr_list, "\n";
    }
  }
my $filename = $ARGV[0];
open(FH, '>', $filename) or die $!;
print FH @illegal_instr_list;
close(FH);
"""



'''
    Module to generate the arguments for the instructions
'''
import aapg.isa_funcs
import aapg.mappings
import random
import os
import logging
import sys

logger = logging.getLogger(__name__)

def set_seed_args_gen(seed):
    """ Set the global seed """
    random.seed(seed)

def gen_branch_args(instruction, regfile, arch, *args, **kwargs):
    """ Generate the args for branch instructions """
    instr_name = instruction[0]
    # Forward or backward jump
    backward = random.random() < float(kwargs['bwd_prob'])

    # Taken or not taken
    taken = random.random() < 0.5

    usable_regs = []
    for (j,i) in regfile:
        if j =="x" and i>15:
            usable_regs.append(i)

    # Don't touch registers used elsewhere in aapg
    if 10 in usable_regs:
        usable_regs.remove(10)
    if 6 in usable_regs:
        usable_regs.remove(6)
    if 5 in usable_regs:
        usable_regs.remove(5)
    if 11 in usable_regs:
        usable_regs.remove(11)
    if 12 in usable_regs:
        usable_regs.remove(12)
    if 13 in usable_regs:
        usable_regs.remove(13)
    if 30 in usable_regs:
        usable_regs.remove(30)
    if 1 in usable_regs:
        usable_regs.remove(1)
    if 0 in usable_regs:
        usable_regs.remove(0)
    if 15 in usable_regs:
        usable_regs.remove(15)

    # Find register for comparison with branch 
    try:
        comp_reg = random.choice(usable_regs)
    except:
        logger.error("Unable to find usable register for branch comparison. Remove registers >15 from no use registers. Exitting")
        sys.exit(1)

    # Number of steps to jump
    try:
        num_steps = random.randint(5, kwargs['insts_since'] - 5)
    except ValueError as e:
        logger.error("Number of instructions required for branch threshold to low")
        logger.error("Increase count of non-branch instructions")
        sys.exit(1)

    # if forward and number of insts not sufficient, switch to bwd
    if not backward and num_steps > kwargs['insts_left'] - 5:
        backward = True

    # Generate the args
    offset_string = '{0},{1}'.format('b' if backward else 'f', num_steps)

    if instr_name == 'beq' or instr_name == 'c.beqz':
        pre_insts = []
        if backward:
            if taken:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '11'])
                pre_insts.append(['addi', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), '1'])
            else:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '0'])
        else:
            if taken:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '10'])
            else:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '0'])
        pre_insts.append(['pre_branch_macro'])
        pre_insts.append(['beq', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), 'x{comp_reg}'.format(comp_reg=comp_reg), offset_string]) 
        pre_insts.append(['post_branch_macro'])
        pre_insts.append(['li', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), '10'])

        pre_insts.append(offset_string)
        return (pre_insts,comp_reg)
    elif instr_name == 'bne' or instr_name == 'c.bnez':
        pre_insts = []
        if backward:
            if taken:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '12'])
                pre_insts.append(['addi', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), '1'])
            else:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '10'])
        else:
            if taken:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '11'])
                pre_insts.append(['li', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), '10']) 
            else:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '10'])
                pre_insts.append(['li', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), '10'])
        pre_insts.append(['pre_branch_macro'])
        pre_insts.append(['bne', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), 'x{comp_reg}'.format(comp_reg=comp_reg), offset_string]) 
        pre_insts.append(['post_branch_macro'])
        pre_insts.append(['li', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), '10'])

        pre_insts.append(offset_string)
        return (pre_insts,comp_reg)
    elif instr_name == 'blt' or instr_name == 'bltu':
        pre_insts = []
        if backward:
            if taken:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '12'])
                pre_insts.append(['addi', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), '1'])
            else:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '9'])
        else:
            if taken:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '12'])
                pre_insts.append(['li', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), '10']) 
            else:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '9'])
                pre_insts.append(['li', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), '10'])
        pre_insts.append(['pre_branch_macro'])
        pre_insts.append(['blt', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), 'x{comp_reg}'.format(comp_reg=comp_reg), offset_string]) 
        pre_insts.append(['post_branch_macro'])
        pre_insts.append(['li', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), '10'])

        pre_insts.append(offset_string)
        return (pre_insts,comp_reg)
    elif instr_name == 'bge' or instr_name == 'bgeu':
        pre_insts = []
        if backward:
            if taken:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '12'])
                pre_insts.append(['addi', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), '1'])
            else:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '9'])
        else:
            if taken:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '11'])
                pre_insts.append(['li', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), '10']) 
            else:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '9'])
                pre_insts.append(['li', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), '10'])
        pre_insts.append(['pre_branch_macro'])
        pre_insts.append(['bge', 'x{comp_reg}'.format(comp_reg=comp_reg), 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), offset_string]) 
        pre_insts.append(['post_branch_macro'])
        pre_insts.append(['li', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), '10'])
        pre_insts.append(offset_string)
        return (pre_insts,comp_reg)
    elif instr_name == 'jal' or instr_name == 'c.j' or instr_name == 'c.jal':
        pre_insts = []

        target_reg = 'x10'

        if instr_name == 'c.j':
            target_reg = 'x0'
        elif instr_name == 'c.jal':
            target_reg = 'x1'

        if backward:
            if taken:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '12'])
                pre_insts.append(['addi', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), '1'])
                pre_insts.append(['beq', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), 'x{comp_reg}'.format(comp_reg=comp_reg), '1f'])
                pre_insts.append(['jal', '{}'.format(target_reg), offset_string])
                pre_insts.append(['1: li x{branch_use_reg}, 10'.format(branch_use_reg=kwargs['branch_use_reg'])])
            else:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '10'])
                pre_insts.append(['beq', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), 'x{comp_reg}'.format(comp_reg=comp_reg), '1f'])
                pre_insts.append(['jal', '{}'.format(target_reg), offset_string])
                pre_insts.append(['1: li x{branch_use_reg}, 10'.format(branch_use_reg=kwargs['branch_use_reg'])])
        else:
            pre_insts.append(['jal', '{}'.format(target_reg), offset_string])
        pre_insts.append(offset_string)
        return (pre_insts,comp_reg)
    elif instr_name == 'jalr' or instr_name == 'c.jr' or instr_name == 'c.jalr':
        pre_insts = []

        target_reg = 'x10'

        if instr_name == 'c.jr':
            target_reg = 'x0'
        elif instr_name == 'c.jalr':
            target_reg = 'x1'

        if backward:
            if taken:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '12'])
                pre_insts.append(['addi', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), '1'])
                pre_insts.append(['beq', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), 'x{comp_reg}'.format(comp_reg=comp_reg), '1f'])
                pre_insts.append(['la', 'x{comp_reg}'.format(comp_reg=comp_reg), offset_string])
                pre_insts.append(['jalr', '{}'.format(target_reg), 'x{comp_reg}'.format(comp_reg=comp_reg), '0'])
                pre_insts.append(['1: li x{branch_use_reg}, 10'.format(branch_use_reg=kwargs['branch_use_reg'])])
            else:
                pre_insts.append(['li', 'x{comp_reg}'.format(comp_reg=comp_reg), '10'])
                pre_insts.append(['beq', 'x{branch_use_reg}'.format(branch_use_reg=kwargs['branch_use_reg']), 'x{comp_reg}'.format(comp_reg=comp_reg), '1f'])
                pre_insts.append(['la', 'x{comp_reg}'.format(comp_reg=comp_reg), offset_string])
                pre_insts.append(['jalr', '{}'.format(target_reg), 'x{comp_reg}'.format(comp_reg=comp_reg), '0'])
                pre_insts.append(['1: li x{branch_use_reg}, 10'.format(branch_use_reg=kwargs['branch_use_reg'])])
        else:
            pre_insts.append(['la', 'x{comp_reg}'.format(comp_reg=comp_reg), offset_string])
            pre_insts.append(['jalr', '{}'.format(target_reg), 'x{comp_reg}'.format(comp_reg=comp_reg), '0'])
        pre_insts.append(offset_string)
        return (pre_insts,comp_reg)
    else:
        return ([['addi', 'x0', 'x0', '0'], 'f,20'],None)

def gen_memory_inst_offset(instr_name):
    """ Generate load/store offset for the instruction """
    if instr_name in ['ld', 'sd', 'fld', 'fsd']:
        return random.choice(range(-2048, 2047, 8))
    if instr_name in ['lw', 'lwu', 'sw', 'flw', 'fsw']:
        return random.choice(range(-2048, 2047, 4))
    if instr_name in ['lh', 'lhu', 'sh']:
        return random.choice(range(-2048, 2047, 2))
    if instr_name in ['lb', 'lbu', 'sb']:
        return random.choice(range(-2048, 2047))

def incr_rw(reg_tup, read, write):
    fst = reg_tup[0]
    snd = reg_tup[1]

    if read == True:
        fst += 1
    if write == True:
        snd += 1

    return (fst, snd)

def gen_args(instruction, regfile, arch, reg_ignore, csr_sections, *args, **kwargs):
    '''
        Function to generate the args for a given instruction

        Args:
            instruction: tuple of (inst_name, *args)
            regfile: value of the registers

        Returns:
            instruction_with_args: (inst_name, arg_strings)
    '''
    instr_name = instruction[0]
    instr_args = instruction[1:]
    # Creating the registers
    register_mapping = aapg.mappings.register_mapping_int
    register_mapping_float = aapg.mappings.register_mapping_float

    # Filter the registers based on raw / war / waw deps
    data_hazards_dict = {x[0] : x[1] for x in kwargs['data_hazards']}
    num_regs = int(kwargs['data_hazards'][3][1])

    # Generate the highest read and highest written thresholds
    take_reg_set = lambda reg_set, regfile : {
            x : regfile[x] for x in regfile if x[0] == reg_set
    }

    take_reg_set_cmp = lambda reg_set, regfile : {
            x : regfile[x] for x in regfile if x[0] == reg_set and x[1] in range(8,16)}

    reg_reads = lambda reg_set, regfile : [
            regfile[x][0] for x in regfile if x[0] == reg_set]
    reg_writes = lambda reg_set, regfile : [
            regfile[x][1] for x in regfile if x[0] == reg_set]

    reg_reads_filt = lambda reg_set, regfile, thresh : {
            x : regfile[x] for x in regfile if x[0] == reg_set
            and
            regfile[x][0] >= thresh}

    reg_writes_filt = lambda reg_set, regfile, thresh : {
            x : regfile[x] for x in regfile if x[0] == reg_set
            and
            regfile[x][1] >= thresh}

    def inverse(reg_set, regfile):
        reg_ret = {x : regfile[x] for x in regfile if x not in reg_set}
        if len(reg_ret.items()) == 0:
            return regfile
        else:
            return reg_ret

    intersect = lambda a, b : [
            x for x in a if x in b
    ]

    max_read_int_thresh = sorted(reg_reads('x', regfile))[-num_regs] 
    max_read_flt_thresh = sorted(reg_reads('f', regfile))[-num_regs]
    max_read_regs_int = reg_reads_filt('x', regfile, max_read_int_thresh)
    max_read_regs_flt = reg_reads_filt('f', regfile, max_read_flt_thresh)

    max_write_int_thresh = sorted(reg_writes('x', regfile))[-num_regs] 
    max_write_flt_thresh = sorted(reg_writes('f', regfile))[-num_regs] 
    max_write_regs_int = reg_writes_filt('x', regfile, max_write_int_thresh)
    max_write_regs_flt = reg_writes_filt('f', regfile, max_write_flt_thresh)

    # Create the empty registers
    registers_cmp_src = []
    registers_cmp_dst = []

    registers_cmp_flt_src = []
    registers_cmp_flt_dst = []

    registers_src = []
    registers_dst = []

    registers_flt_src = []
    registers_flt_dst = []

    registers_comp = [x for x in regfile if x[1] in range(8,16) and x[0] == 'x']

    registers_comp_float = [x for x in regfile if x[1] in range(8,16) and x[0] == 'f']
    registers_int = [x for x in regfile if x[0] == 'x'] 
    registers_float = [x for x in regfile if x[0] == 'f']

    if random.random() < float(data_hazards_dict['raw_prob']):
        # Generate RAW Hazard. read registers should be
        # the highest written to registers
        registers_src.extend(take_reg_set('x', max_write_regs_int))
        registers_flt_src.extend(take_reg_set('f', max_write_regs_flt))
        registers_cmp_src.extend(take_reg_set_cmp('x', max_write_regs_int))
        registers_cmp_flt_src.extend(take_reg_set_cmp('f', max_write_regs_flt))
    else:
        # RAW not generated, src registers should be the inverse of 
        # max read regs
        registers_src.extend(take_reg_set('x', inverse(max_write_regs_int, regfile)))
        registers_flt_src.extend(take_reg_set('f', inverse(max_write_regs_flt, regfile)))
        registers_cmp_src.extend(take_reg_set_cmp('x', inverse(max_write_regs_int, regfile)))
        registers_cmp_flt_src.extend(take_reg_set_cmp('f', inverse(max_write_regs_flt, regfile)))

    if random.random() < float(data_hazards_dict['war_prob']):
        # Generate WAR hazard. write registers should be
        # the highest read registers
        registers_dst.extend(take_reg_set('x', max_read_regs_int))
        registers_flt_dst.extend(take_reg_set('f', max_read_regs_flt))
        registers_cmp_dst.extend(take_reg_set_cmp('x', max_read_regs_int))
        registers_cmp_flt_dst.extend(take_reg_set_cmp('f', max_read_regs_flt))
    else:
        # WAR not generated. dst registers should be inverse of
        # max read regs
        registers_dst.extend(take_reg_set('x', inverse(max_read_regs_int, regfile)))
        registers_flt_dst.extend(take_reg_set('f', inverse(max_read_regs_flt, regfile)))
        registers_cmp_dst.extend(take_reg_set_cmp('x', inverse(max_read_regs_int, regfile)))
        registers_cmp_flt_dst.extend(take_reg_set_cmp('f', inverse(max_read_regs_flt, regfile)))

    if random.random() < float(data_hazards_dict['waw_prob']):
        # Generate WAW hazard. write registers should be
        # the highest written to registers
        registers_dst.extend(intersect(registers_dst, max_write_regs_int))
        registers_flt_dst.extend(intersect(registers_flt_dst, max_write_regs_flt))
        registers_cmp_dst.extend(intersect(registers_cmp_dst, max_write_regs_int))
        registers_cmp_flt_dst.extend(intersect(registers_cmp_flt_dst, max_write_regs_flt))
    else:
        # WAW not generated, intersect with inverse
        registers_dst.extend(intersect(registers_dst, inverse(max_write_regs_int, regfile)))
        registers_flt_dst.extend(intersect(registers_flt_dst, inverse(max_write_regs_flt, regfile)))
        registers_cmp_dst.extend(intersect(registers_cmp_dst, inverse(max_write_regs_int, regfile)))
        registers_cmp_flt_dst.extend(intersect(registers_cmp_flt_dst, inverse(max_write_regs_flt, regfile)))

    # Iterate over the args
    final_inst = [instr_name,]

    for arg in instr_args:
        if arg == 'rd':
            try:
                register = random.choice(registers_dst)
            except IndexError as e:
                register = random.choice(registers_int)
            regfile[register] = incr_rw(regfile[register], False, True)
            final_inst.append(register_mapping[register])
            continue

        if arg == 'rs1':
            try:
                register = random.choice(registers_src)
            except IndexError as e:
                register = random.choice(registers_int)
            regfile[register] = incr_rw(regfile[register], True, False)
            final_inst.append(register_mapping[register])
            continue

        if arg == 'rs2':
            try:
                register = random.choice(registers_src)
            except IndexError as e:
                register = random.choice(registers_int)
            regfile[register] = incr_rw(regfile[register], True, False)
            final_inst.append(register_mapping[register])
            continue

        if arg in ['imm12', 'uimm12']:

            # Check if memory_inst
            if instr_name in aapg.isa_funcs.memory_insts:
                imm12_val = gen_memory_inst_offset(instr_name)

            else:
                if arg == 'uimm12':
                    if "csr" in instr_name:
                        # Control CSR Access based on config file

                        # Don't write to xstatus, xtvec, xepc, xcause
                        csr_no_use = [0,5,65,66,256,261,321,322,768,773,833,834]
                        valid_ranges = csr_sections.split(',')
                        valid_csrs = []
                        for item in valid_ranges:
                            item = item.replace(' ','')
                            if ':' in item:
                                begin,end = item.split(':')
                                begin = int(begin, 16)
                                end = int(end,16)
                                for int_val in range(begin,end+1):
                                    if int_val not in csr_no_use:
                                        valid_csrs.append(int_val)
                            else:
                                valid_csrs.append(int(item,16))
                        imm12_val = random.choice(valid_csrs)
                    else:
                        imm12_val = random.randint(0,4095)
                else:
                    imm12_val = random.randint(-2048, 2047)

            final_inst.append(str(imm12_val))
            continue

        if arg == 'imm5':
            final_inst.append(str(random.randint(-16, 15)))
            continue

        if arg == 'uimm5':
            final_inst.append(str(random.randint(0,31)))
            continue

        if arg == 'uimm6':
            final_inst.append(str(random.randint(0,63)))
            continue

        if arg == 'imm6':
            final_inst.append(str(random.randint(- 1<<5, 1<<5 - 1)))

        if arg == 'imm20':
            final_inst.append(str(random.randint(0, 1<<20 - 1)))
            continue

        if arg == 'shamt':
            if arch == 'rv32':
                final_inst.append(str(random.randint(0, 31)))
            elif arch == 'rv64':
                final_inst.append(str(random.randint(0, 63)))
            continue

        if arg == 'shamtw':
            final_inst.append(str(random.randint(0, 31)))
            continue

        # Floating point instructions
        if arg == 'rdf':
            try:
                register = random.choice(registers_flt_dst)
            except IndexError as e:
                register = random.choice(registers_float)
            regfile[register] = incr_rw(regfile[register], False, True)
            final_inst.append(register_mapping_float[register])
            continue

        if arg == 'rs1f':
            try:
                register = random.choice(registers_flt_src)
            except IndexError as e:
                register = random.choice(registers_float)
            regfile[register] = incr_rw(regfile[register], True, False)
            final_inst.append(register_mapping_float[register])
            continue

        if arg == 'rs2f':
            try:
                register = random.choice(registers_flt_src)
            except IndexError as e:
                register = random.choice(registers_float)
            regfile[register] = incr_rw(regfile[register], True, False)
            final_inst.append(register_mapping_float[register])
            continue

        if arg == 'rs3f':
            try:
                register = random.choice(registers_flt_src)
            except IndexError as e:
                register = random.choice(registers_float)
            regfile[register] = incr_rw(regfile[register], True, False)
            final_inst.append(register_mapping_float[register])
            continue

        # Compressed
        if arg == 'rdprime':
            try:
                register = random.choice(registers_cmp_dst)
            except IndexError as e:
                register = random.choice(registers_comp)
            regfile[register] = incr_rw(regfile[register], False, True)
            final_inst.append(register_mapping[register])
            continue

        if arg == 'rsprime1':
            try:
                register = random.choice(registers_cmp_src)
            except IndexError as e:
                register = random.choice(registers_comp)
            regfile[register] = incr_rw(regfile[register], True, False)
            final_inst.append(register_mapping[register])
            continue

        if arg == 'rsprime2':
            try:
                register = random.choice(registers_cmp_src)
            except IndexError as e:
                register = random.choice(registers_comp)
            regfile[register] = incr_rw(regfile[register], True, False)
            final_inst.append(register_mapping[register])
            continue

        if arg == 'nzuimm8':
            final_inst.append(str(4*random.randint(0, 1<<8 - 1)))
            continue

        if arg == 's8uimm6':
            final_inst.append(str(8*random.randint(0, 1<<6 - 1)))
            continue

        if arg == 's4uimm6':
            final_inst.append(str(4*random.randint(0, 1<<6 - 1)))
            continue

        if arg == 'imm8':
            final_inst.append(str(random.randint(- 1 << 7, 1 << 7 - 1)))
            continue

        if arg == 'nzimm6':
            final_inst.append(str(random.randint(- 1<<5, 1<<5 - 1)))
            continue

        if arg == 'nzuimm6':
            final_inst.append(str(random.randint(1, 1<<6 - 1)))
            continue

        if arg == 'nzuimm5':
            final_inst.append(str(random.randint(1, 1 << 5 - 1)))
            continue

        if arg == 's16imm6':
            final_inst.append((str(16*random.randint(-1 <<5, 1<<5 - 1))))
            continue

        if arg == 'sp':
            final_inst.append('sp')
            continue

        if arg == 'a0':
            final_inst.append('a0')
            continue

        if arg == 's8uimm5':
            final_inst.append(str(8*random.randint(0, 1<<5 -1)))
            continue 

        if arg == 's4uimm5':
            final_inst.append(str(4*random.randint(0, 1<<5 -1)))
            continue

        if arg == 'rdprimef':
            try:
                register = random.choice(registers_cmp_flt_dst)
            except IndexError as e:
                register = random.choice(registers_comp_float)
            regfile[register] = incr_rw(regfile[register], False, True)
            final_inst.append(register_mapping_float[register])
            continue

        if arg == 'rsprime1f':
            try:
                register = random.choice(registers_cmp_flt_src)
            except IndexError as e:
                register = random.choice(registers_comp_float)
            regfile[register] = incr_rw(regfile[register], True, False)
            final_inst.append(register_mapping_float[register])
            continue

        if arg == 'rsprime2f':
            try:
                register = random.choice(registers_cmp_flt_src)
            except IndexError as e:
                register = random.choice(registers_comp_float)
            regfile[register] = incr_rw(regfile[register], True, False)
            final_inst.append(register_mapping_float[register])
            continue

        if arg == 'rd_rs1_prime':
            try:
                register = random.choice(registers_cmp_dst)
            except IndexError as e:
                register = random.choice(registers_comp)
            regfile[register] = incr_rw(regfile[register], True, True)
            reg_map = register_mapping[register]
            final_inst.append(reg_map)
            final_inst.append(reg_map)
            continue

        if arg == 'rd_rs1':
            try:
                register = random.choice(registers_dst)
            except IndexError as e:
                register = random.choice(registers_int)
            regfile[register] = incr_rw(regfile[register], True, True)
            reg_map = register_mapping[register]
            final_inst.append(reg_map)
            final_inst.append(reg_map)
            continue

        if arg == 'imm11':
            final_inst.append(str(random.randint(- 1<<10, 1<<10 - 1)))
            continue

        if arg == 'x1':
            final_inst.append(register_mapping[('x', 1)])
            continue
        
        if arg == 'x0':
            final_inst.append(register_mapping[('x', 0)])
            continue

        if arg == 'const0':
            final_inst.append('0')
            continue

    if final_inst[0] in aapg.isa_funcs.comp_insts:
        final_inst[0] = aapg.isa_funcs.comp_insts_subs[final_inst[0]]
        
    return tuple(final_inst)

def gen_atomic_args(instruction, regfile, arch, reg_ignore, *args, **kwargs):
    """ Generate args for atomic insts"""

    instr_name = instruction[0]
    instr_args = instruction[1:]

    # Creating the registers
    registers_int = [x for x in regfile if x[0] == 'x'] 
    register_mapping = aapg.mappings.register_mapping_int
    registers_src = []
    registers_dst = []
    
    # Filter the registers based on raw / war / waw deps
    data_hazards_dict = {x[0] : x[1] for x in kwargs['data_hazards']}
    num_regs = int(kwargs['data_hazards'][3][1])

    # Generate the highest read and highest written thresholds
    take_reg_set = lambda reg_set, regfile : {
            x : regfile[x] for x in regfile if x[0] == reg_set
    }

    take_reg_set_cmp = lambda reg_set, regfile : {
            x : regfile[x] for x in regfile if x[0] == reg_set and x[1] in range(8,16)}

    reg_reads = lambda reg_set, regfile : [
            regfile[x][0] for x in regfile if x[0] == reg_set]
    reg_writes = lambda reg_set, regfile : [
            regfile[x][1] for x in regfile if x[0] == reg_set]

    reg_reads_filt = lambda reg_set, regfile, thresh : {
            x : regfile[x] for x in regfile if x[0] == reg_set
            and
            regfile[x][0] >= thresh}

    reg_writes_filt = lambda reg_set, regfile, thresh : {
            x : regfile[x] for x in regfile if x[0] == reg_set
            and
            regfile[x][1] >= thresh}

    def inverse(reg_set, regfile):
        reg_ret = {x : regfile[x] for x in regfile if x not in reg_set}
        if len(reg_ret.items()) == 0:
            return regfile
        else:
            return reg_ret

    intersect = lambda a, b : [
            x for x in a if x in b
    ]

    max_read_int_thresh = sorted(reg_reads('x', regfile))[-num_regs] 
    max_read_regs_int = reg_reads_filt('x', regfile, max_read_int_thresh)

    max_write_int_thresh = sorted(reg_writes('x', regfile))[-num_regs] 
    max_write_regs_int = reg_writes_filt('x', regfile, max_write_int_thresh)

    # Data hazards
    if random.random() < float(data_hazards_dict['raw_prob']):
        # Generate RAW Hazard. read registers should be
        # the highest written to registers
        registers_src.extend(take_reg_set('x', max_write_regs_int))
    else:
        # RAW not generated, src registers should be the inverse of 
        # max read regs
        registers_src.extend(take_reg_set('x', inverse(max_write_regs_int, regfile)))

    if random.random() < float(data_hazards_dict['war_prob']):
        # Generate WAR hazard. write registers should be
        # the highest read registers
        registers_dst.extend(take_reg_set('x', max_read_regs_int))
    else:
        # WAR not generated. dst registers should be inverse of
        # max read regs
        registers_dst.extend(take_reg_set('x', inverse(max_read_regs_int, regfile)))

    if random.random() < float(data_hazards_dict['waw_prob']):
        # Generate WAW hazard. write registers should be
        # the highest written to registers
        registers_dst.extend(intersect(registers_dst, max_write_regs_int))
    else:
        # WAW not generated, intersect with inverse
        registers_dst.extend(intersect(registers_dst, inverse(max_write_regs_int, regfile)))

    final_inst = [instr_name,]

    # Updating the args
    for arg in instr_args:

        if arg == 'rd':
            try:
                register = random.choice(registers_dst)
            except IndexError as e:
                register = random.choice(registers_int)
            regfile[register] = incr_rw(regfile[register], False, True)
            final_inst.append(register_mapping[register])

        if arg == 'rs1':
            if instr_name == 'lr.w' or instr_name == 'lr.d':
                final_inst.append('sp')
            else:
                try:
                    register = random.choice(registers_src)
                except IndexError as e:
                    register = random.choice(registers_int)
                regfile[register] = incr_rw(regfile[register], True, False)
                final_inst.append(register_mapping[register])

        if arg == 'rs2':
            final_inst.append('sp')

    # Iterate over the args

    return final_inst



"""
    Helpers for different functions
"""
import logging
import aapg.env.config
import os

class ColoredFormatter(logging.Formatter):
    """
        Class to create a log output which is colored based on level.
    """
    def __init__(self, *args, **kwargs):
        super(ColoredFormatter, self).__init__(*args, **kwargs)
        self.colors = {
                'DEBUG' : '\033[94m',
                'INFO'  : '\033[92m',
                'WARNING' : '\033[93m',
                'ERROR' : '\033[91m',
        }

        self.reset = '\033[0m'

    def format(self, record):
        msg = str(record.msg)
        level_name = str(record.levelname)
        name = str(record.name)
        color_prefix = self.colors[level_name]
        return '{0}{1:<9s} - {2:<25s} : {3}{4}'.format(
                color_prefix,
                '[' + level_name + ']',
                name,
                msg,
                self.reset)

def print_sample_config(output_dir = '.'):
    """
        Print sample config.ini
    """
    config_sample = aapg.env.config.template_config
    output_abspath = os.path.abspath(output_dir)
    output_file = os.path.join(output_abspath, 'config.yaml')
    with open(output_file, 'w') as f:
        f.write(config_sample.strip('\n'))
